package org.jcg.zheng;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import org.junit.Test;

// Since Java 1.0
public class System_arraycopyTest extends TestBase {

    private int length = 3;

    @Test
    public void copy_IntArray() {
        int[] destination = new int[length];

        System.arraycopy(sourceIntegerArray, copyStartAtZeroIndex, destination, copyStartAtZeroIndex, length);

        assertEquals(length, destination.length);
        assertEquals(sourceIntegerArray[copyStartAtZeroIndex], destination[0]);
        assertEquals(sourceIntegerArray[copyStartAtZeroIndex + 1], destination[1]);
        assertEquals(sourceIntegerArray[copyStartAtZeroIndex + 2], destination[2]);

        validate_IntArray(destination);
    }

    @Test
    public void copy_2D_IntArray() {
        int[][] clonedArray = new int[source2DArray.length][];
        for (int i = 0; i < source2DArray.length; i++) {
            int[] aSource = source2DArray[i];
            clonedArray[i] = new int[source2DArray.length];
            System.arraycopy(aSource, 0, clonedArray[i], 0, source2DArray.length);
        }

        assertEquals(source2DArray.length, clonedArray.length);
        assertEquals(source2DArray[0][0], clonedArray[0][0]);
        assertEquals(source2DArray[0][1], clonedArray[0][1]);
        assertEquals(source2DArray[1][0], clonedArray[1][0]);
        assertEquals(source2DArray[1][1], clonedArray[1][1]);
    }

    @Test
    public void copy_StringArray() {
        String[] destination = new String[length];

        System.arraycopy(sourceStringArray, copyStartAtZeroIndex, destination, copyStartAtZeroIndex, length);

        assertEquals(sourceStringArray.length, destination.length);
        assertEquals(sourceStringArray[0], destination[0]);
        assertEquals(sourceStringArray[1], destination[1]);

        validate_StringArray(destination);
    }

    @Test
    public void copy_ObjectArray() {
        DemoObject[] destination = new DemoObject[length];

        System.arraycopy(sourceObjectArray, copyStartAtZeroIndex, destination, copyStartAtZeroIndex, length);

        assertEquals(length, destination.length);
        validate_shallowCopy_ObjectArray(destination);
    }

    @Test
    public void padding_null_when_destination_is_bigger_than_original() {
        DemoObject[] destination = new DemoObject[5];

        System.arraycopy(sourceObjectArray, copyStartAtZeroIndex, destination, copyStartAtZeroIndex,
                sourceObjectArray.length);

        assertNull(destination[sourceObjectArray.length]);
        assertNull(destination[sourceObjectArray.length + 1]);
    }

    @Test(expected = NullPointerException.class)
    public void exception_when_destination_is_null() {
        System.arraycopy(sourceObjectArray, copyStartAtZeroIndex, null, copyStartAtZeroIndex,
                sourceObjectArray.length);
    }

    @Test(expected = ArrayIndexOutOfBoundsException.class)
    public void exception_when_copy_lenght_exceed_destination() {
        DemoObject[] destination = new DemoObject[1];
        System.arraycopy(sourceObjectArray, copyStartAtZeroIndex, destination, copyStartAtZeroIndex,
                destination.length + 1);
    }

    @Test(expected = ArrayIndexOutOfBoundsException.class)
    public void exception_when_copy_length_exceed_source() {
        DemoObject[] destination = new DemoObject[5];
        System.arraycopy(sourceObjectArray, copyStartAtZeroIndex, destination, copyStartAtZeroIndex,
                sourceObjectArray.length + 1);
    }

}