package org.jcg.zheng;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

public class TestBase {

    protected int[] sourceIntegerArray = { 1, 2, 3, 4, 5 };
    protected String[] sourceStringArray = { "Mary", "Zheng", "Test" };

    protected int[][] source2DArray = { sourceIntegerArray, { 3, 4 } };
    protected DemoObject maryObject = new DemoObject('A', 1, "Mary");
    protected DemoObject zhengObject = new DemoObject('B', 2, "Zheng");
    protected DemoObject[] sourceObjectArray = { maryObject, zhengObject, null };
    protected DemoObject[] sourceObjectArrayContainsNull = { maryObject, null };

    protected int copyStartAtZeroIndex = 0;

    protected void validate_IntArray(int[] clonedArray) {
        // the copied object is a different reference
        assertFalse(System.identityHashCode(sourceIntegerArray)
                - System.identityHashCode(clonedArray) == 0);

        // for primitive data type, change the clonedArray element does not affect the source array
        clonedArray[0] = 99;

        assertEquals(99, clonedArray[0]);
        assertEquals(1, sourceIntegerArray[0]);

        sourceIntegerArray[1] = 99;
        assertEquals(2, clonedArray[1]);
    }

    protected void validate_StringArray(String[] clonedArray) {
        // the copied object is a different reference
        assertFalse(System.identityHashCode(sourceStringArray)
                - System.identityHashCode(clonedArray) == 0);

        // change the clonedArray element does not affect the source array
        clonedArray[0] = "Dummy";

        assertEquals("Dummy", clonedArray[0]);
        assertEquals("Mary", sourceStringArray[0]);

        sourceStringArray[1] = "ZhengUpdate";
        assertEquals("Zheng", clonedArray[1]);
    }

    protected void validate_shallowCopy_ObjectArray(DemoObject[] clonedArray) {
        // the copied object is a different reference
        assertFalse(System.identityHashCode(sourceObjectArray)
                - System.identityHashCode(clonedArray) == 0);

        // the copied object element is the same reference
        assertTrue(
                System.identityHashCode(maryObject) - System.identityHashCode(clonedArray[0]) == 0);
        assertTrue(System.identityHashCode(zhengObject)
                - System.identityHashCode(clonedArray[1]) == 0);

        assertEquals(maryObject, clonedArray[0]);
        assertEquals(zhengObject, clonedArray[1]);

        // for the copied object, change one will affect the other due to object reference is same
        clonedArray[1].setName("Changed");
        assertEquals("Changed", sourceObjectArray[1].getName());

        sourceObjectArray[1].setNumber(99);
        assertEquals(99, clonedArray[1].getNumber());

    }

    protected void validate_deepCopy_ObjectArray(DemoObject[] clonedArray) {
        // the copied object is a different reference
        assertFalse(System.identityHashCode(sourceObjectArray)
                - System.identityHashCode(clonedArray) == 0);

        assertEquals(3, clonedArray.length);

        // the object element reference is different
        assertFalse(
                System.identityHashCode(maryObject) - System.identityHashCode(clonedArray[0]) == 0);
        assertFalse(System.identityHashCode(zhengObject)
                - System.identityHashCode(clonedArray[1]) == 0);

        assertEquals(maryObject, clonedArray[0]);
        assertEquals(zhengObject, clonedArray[1]);

        // the object change will NOT affect the other because of deep copy
        clonedArray[1].setName("Changed");
        assertEquals("Zheng", sourceObjectArray[1].getName());

        sourceObjectArray[1].setNumber(99);
        assertEquals(2, clonedArray[1].getNumber());
    }

}